/*
 * $Id: ReadCSV.java,v 1.3 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package fr.fluxmedia.tmcontrib.generator;

// Imported TRANSMORPHER Classes
import fr.fluxmedia.transmorpher.engine.TReader;
import fr.fluxmedia.transmorpher.engine.XML_Port;
import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.StringParameters;
//Imported JAVA classes
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.*;
// Imported SAX Classes
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Read a CSV (comma separated value) file which coordinates (local path or URL) are given in the
 * file parameter of the generate call, parse it, process it through CSVXMLReader
 * and generate a SAX event flow to the next handler
 *
 *
 *@author    triolet
 */
public class ReadCSV extends TReader {

	/**
	 *Constructor for the readCSV object
	 *
	 *@param  pOut               The out port of this generator
	 *@param  pParam             The parameters of this generator
	 *@param  pStaticAttributes  The parameters of this generator
	 *@exception  SAXException   an exception that an wrap others exceptions
	 *@exception  IOException    if an I/O operation failed
	 */
	public ReadCSV(String[] pOut, Parameters pParam, StringParameters pStaticAttributes) throws SAXException, IOException {
		super(pOut, pParam, pStaticAttributes);

		iReader = new CSVXMLReader(((String)pParam.getParameter("separator")));
	}

	/**
	 * Creates an input source for the parser with the name of the file and starts
	 * the parsing
	 *
	 *@exception  IOException   if an I/O operation failed
	 *@exception  SAXException  an exception that an wrap others exceptions
	 */
	public void read() throws IOException, SAXException {
		try {
			InputSource inSource;
			if (iFilename != null) {
				if (iFilename.startsWith("http://")) {
					URL url = new URL(iFilename);
					URLConnection urlConnect = url.openConnection();
					InputStream inStream = urlConnect.getInputStream();
					inSource = new InputSource(inStream);
					inSource.setSystemId(iFilename);
				} else {
					File file = new File(iFilename);
					inSource = new InputSource(new FileInputStream(file));
					inSource.setSystemId(file.toURL().toExternalForm());
				}
				iReader.parse(inSource);
			} else {
				iReader.parse(standardInput);
			}//end if
		} catch (IOException e) {
			System.err.println("File : " + iFilename + " can't be found.");
			iReader.getContentHandler().startDocument();
			iReader.getContentHandler().endDocument();
		}
	}
}

