/**
 * $Id: TTransformation.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
  *
  * This class allow to build a Transformation from JAXP sources
  *
  * @author Jerome Euzenat (Jerome.Euzenat@inrialpes.fr)
  * @since jdk 1.2 / SAX 2.0
*/

package fr.fluxmedia.transmorpher.engine ;

// Imported JAXP classes

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.Templates;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.sax.SAXResult;

// Imported SAX classes

import org.xml.sax.SAXException;

// Imported java.io classes

import java.io.IOException;
import java.io.File;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Map;
import java.util.Enumeration;

// Imported transmorpher classes

import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.TMException;
import fr.fluxmedia.transmorpher.utils.TMRuntimeException;
import fr.fluxmedia.transmorpher.engine.*;

/**
 * During the parse process an xsl file was generated to handle the define external.
 *So, the external process, is only a transformer
*/
public abstract class TTransformation extends TProcessBasic {
    
    /** port name in */
    String[] In = null;
    
    /** port name out */
    String[] Out = null;
	
    /** the templates */
    Templates templates = null;

    /** The transformer */
    TransformerHandler iTransformerHandler = null;
    
    /** Cache for stylesheets*/
    private static Map cache = new HashMap();
    
    /** The file in which the transformation is stored */
    // It would be useful to directly feed this Transformer with Templates...
    // This can be found on p24 of current JAXP 1.1 spec
    protected String fileName = null;
    
    /** the properties for all transformer */
    static java.util.Properties Op = null ;
    
    /** The instance  constructor
    */
    public TTransformation (String[] pIn, String[] pOut, Parameters params)
	throws TransformerException, TransformerConfigurationException,
	       SAXException, IOException {
	super(pIn,pOut,params);
	// This should be used by the factory for creating the class...
	fileName = (String)params.getParameter("file");
    }  
    /**
    * Returns file
    */	
    public String getFile(){
	return fileName;
    }

    /** 
     *create The Transformer Handler 
     */
    public final Transformer initTransformer() throws TMRuntimeException {
	try {
	    File xsltFile=new File(fileName);

	    long xslLastModified = xsltFile.lastModified();
	    MapEntry entry  =(MapEntry) cache.get(fileName);
	    if (entry!=null){
		if(xslLastModified>entry.lastModified) entry=null;
	    }
	    if(entry==null){
		String systemID = xsltFile.toURL().toExternalForm();
		StreamSource SS = new StreamSource(systemID) ;
		// Create a TransformerHandler for each stylesheet.
		templates = TProcessFactory.getSAXTFactory().newTemplates(SS);
		entry=new MapEntry(xslLastModified,templates);
		cache.put(fileName,entry);
	    }
	    templates=entry.templates;
	    iTransformerHandler = TProcessFactory.getSAXTFactory().newTransformerHandler(templates);
	    setOutputProperties();
	} catch (Exception e) {
	    throw new TMRuntimeException(e, "TTransformation initialization failled");
	}
	
	return getTransformer();
    }
    
    /**
    * Sets output properties for this Transformer
    */ 
    public void setOutputProperties(){
  	if (Op == null) {
	    Op = getTransformer().getOutputProperties();
	    // Isn't it UTF-8 everywhere?
	    Op.setProperty(javax.xml.transform.OutputKeys.ENCODING,"UTF-8");
	    Op.setProperty(javax.xml.transform.OutputKeys.INDENT,"yes");
	    Op.setProperty(javax.xml.transform.OutputKeys.METHOD,"xml");
	} //end if
	getTransformer().setOutputProperties(Op);
    }
    
  /*------------------------    getTransformer getTransfomerHandler -----------------*/
    
    /** get The Transformer Handler */
    public final TransformerHandler getTransformerHandler() {
	return iTransformerHandler;
    }
    
    /** get The Transformer Handler */
    public final Transformer getTransformer() {
	return iTransformerHandler.getTransformer();
    }
    
    /**
    * Creates a new TransformerHandler.Used by repeat process at each loop.(Because of a xalan bug, a TransformerHandler can not be used several times. 
    */
    public void reset(){
	try{
		iTransformerHandler=null;
		iTransformerHandler = TProcessFactory.getSAXTFactory().newTransformerHandler(templates);
		setOutputProperties();
	}
	catch(Exception e){
	}    
    }

    /**
    * generates ports for a TTranformer
    */	
    public void generatePort() {
	//System.out.println("[TTransformation] generatePorts()");
	String nameI = nameIn[0];
	String nameO = nameOut[0];
	/*if(nameFather != null)  {
	    nameI = nameFather+"-"+nameI;
	    nameO = nameFather+"-"+nameO;
	}*/
	iListIn [0] = new XML_Port(nameI,this,iTransformerHandler,0,XML_Port.IN);
	iListOut[0] = new XML_Port(nameO,this,iTransformerHandler,0,XML_Port.OUT);
	//System.out.println(nameI+" "+iListIn[0]+" "+nameO+" "+iListOut[0]);

    }

    public void bindParameters( Parameters p ) throws TMRuntimeException, TMException, SAXException {
	super.bindParameters(p);
	if ( getParameter("file") != null ) fileName = (String)getParameter("file");
	initTransformer();
	// Then pass them to the transformer instance
	for( Enumeration e = getParameterNames() ; e.hasMoreElements(); ){
	    String key = (String)e.nextElement();
	    getTransformer().setParameter(key,getParameter(key));
	} //end for
    }
     
     static class MapEntry{
	     long lastModified;
	     Templates templates;
	     
	     MapEntry(long lastModified,Templates templates){
		     this.lastModified=lastModified;
		     this.templates=templates;
	     }
     }
	     
} //end class

