/**
 *  $Id: ApplyExternal.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 * 
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.transmorpher.graph;

import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.engine.TProcess;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;

/**
 *  Transmorpher graph call node interface
 *
 *@author     Jerome.Euzenat@inrialpes.fr
 *@since      jdk 1.3 / SAX 2.0
 */
public class ApplyExternal extends ApplyImpl {

	/**
	 *  Constructor for the ApplyExternal object
	 */
	public ApplyExternal() {
		this((String) null, (String) null);
	}

	/**
	 *  Constructor for the ApplyExternal object
	 *
	 *@param  n  ApplyExternal name
	 *@param  t  ApplyExternal type
	 */
	public ApplyExternal(String n, String t) {
		this(n, t, (String) null);
	}


	/**
	 *  Constructor for the ApplyExternal object
	 *
	 *@param  n  ApplyExternal name
	 *@param  t  ApplyExternal type
	 *@param  f  Description of the Parameter
	 */
	public ApplyExternal(String n, String t, String f) {
		this(n, t, f, null);
	}

	/**
	 *  Constructor for the ApplyExternal object
	 *
	 *@param  n  ApplyExternal name
	 *@param  t  ApplyExternal type
	 *@param  p
	 */
	public ApplyExternal(String n, String t, Process p) {
		this(n, t, null, p);
	}

	/**
	 *  Constructor for the ApplyExternal object
	 *
	 *@param  n  ApplyExternal name
	 *@param  t  ApplyExternal type
	 *@param  p
	 *@param  f  Description of the Parameter
	 */
	public ApplyExternal(String n, String t, String f, Process p) {
		this(n, t, f, p, 0, 0);
	}

	/**
	 *  Constructor for the ApplyExternal object
	 *
	 *@param  n     ApplyExternal name
	 *@param  t     ApplyExternal type
	 *@param  file
	 *@param  p     Description of the Parameter
	 *@param  in    Number of ins
	 *@param  out   Nomber of outs
	 */
	public ApplyExternal(String n, String t, String file, Process p, int in, int out) {
		super(n, t, p, null, in, out);
		if (file != null) {
			parameters.setParameter("file", file);
		}
	}

	/**
	 *  Sets the file attribute of the ApplyExternal object
	 *
	 *@param  uri  The new file value
	 */
	public void setFile(String uri) {
		if (uri != null) {
			parameters.setParameter("file", uri);
		}
	}

	/**
	 *  Generates XML for this component
	 */
	public void generateXML() {
		System.out.print("      <apply-external id=\"" + id + "\" type=\"" + getType() + "\" ");
		System.out.print("in=\"");
		inPorts.generateXML();
		System.out.print("\" out=\"");
		outPorts.generateXML();
		if (parameters.isEmpty()) {
			System.out.println("\"/>");
		} else {
			System.out.println("\">");
			parameters.generateXMLCalls();
			System.out.println("      </apply-external>");
		}//end if
	}//end generate XML

	/**
	 *  Creates a new TApplyProcess corresponding to this ApplyProcess.
	 *
	 *@param  currentProcess   The container of this component.
	 *@param  iProcessFactory  Used to create the TApplyProcess.
	 */
	public final void createProcess(TProcessComposite currentProcess, TProcessFactory iProcessFactory) {
		String[] vPortIn = inPorts().toStringList();
		String[] vPortOut = outPorts().toStringList();
		TProcess vCurrent = (TProcess) TProcessFactory.newExternal(vPortIn, vPortOut,
				getAttributes().getStringParameter("type"),
				getParameters(),
				getAttributes());
		currentProcess.addProcess(vCurrent);
		update(vCurrent);
	}//end proc

	/**
	 *  Generates java code of this component
	 *
	 *@param  file  the file to write in.
	 */
	public void generateJavaCode(Writer file) throws IOException {
	    file.writeln();
	    file.writeln(4, "//------------------ Generation of ApplyExternal " + getId() + " -------------------");
	    generatePorts(file);
	    generateParameters(file);
	    file.writeln(4, "tmCurrentProcess = tmProcessFactory.newExternal(portIn,portOut,\"" + getType() + "\",tmParameters,tmAttributes);");
	    //+",tmAttributes);");
	    file.writeln(4, "tmCurrentProcess.setName(\"" + getId() + "\");");
	    file.writeln();
	    file.writeln(4, "tmCurrentCompositeProcess.addProcess(tmCurrentProcess);");
	}//end proc
}

