/**
 * $Id: ApplyProcess.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph call node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.util.ListIterator;
import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct ;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.engine.TApplyProcess;
import fr.fluxmedia.transmorpher.engine.TProcess;

public class ApplyProcess extends ApplyImpl {
    
    public ApplyProcess(String n, String t){
	this(n,t,null,null);
    }
    public ApplyProcess(String n, String t, String pr){
	this(n,t,null,pr);
    }
    
    public ApplyProcess(String n, String t, Process p){
	this(n,t,p,null,0,0);
    }

    public ApplyProcess(String n, String t, Process p, String pr){
	this(n,t,p,pr,0,0);
    }
    
    
    public ApplyProcess (String n, String t, Process p, String pr, int in, int out){
	super(n, t, p, pr, in, out);
    }
    
    /** ask a nullify call to the called process */
    public void retroNull( PortList out, PortList in) {
	// JE: check is it is a Process
	Process called = (Process)(getProcess().getTransmorpher().findTransformation(getRef()));
	nullified = called.retroNull(out, in, true);
	visited = true;

	update(null);
    }
    
    public void generateXML (){
	System.out.print("      <apply-process id=\""+id+"\" ref=\""+getRef()+"\" ");
	System.out.print("in=\"");
	inPorts.generateXML();
	System.out.print("\" out=\"");
	outPorts.generateXML();
	if ( parameters.isEmpty() ){
	    System.out.println("\"/>");
	} else {
	    System.out.println("\">");
	    parameters.generateXMLCalls();
	    System.out.println("      </apply-process>");
	}
    } //end generate XML
    
    public final void createProcess(TProcessComposite currentProcess, TProcessFactory iProcessFactory){

	String[] vPortIn = inPorts().toStringList();
	String[] vPortOut = outPorts().toStringList();
	// get the name corresponding to the generated stylesheet and pass it in parameter
	String ref = getAttributes().getStringParameter("ref");
	Transmorpher t = getProcess().getTransmorpher() ;
	Process p = (Process)(t.findTransformation(ref));
	LinearIndexedStruct vCalls = p.getCalls();
	//System.out.println("[ExecutionStructure]ApplyProcess : findTransformation"
	//                    +((fr.fluxmedia.transmorpher.graph.Process)iTransmorpher.getTransformation(ref)));
	TProcessComposite vCurrent =
	    (TProcessComposite)TProcessFactory.newApplyProcess(vPortIn,vPortOut,
							       getAttributes().getStringParameter("type"),
							       getParameters(),
							       getAttributes());
	PortList LocIn = p.inPorts();
	PortList LocOut= p.outPorts();
	((TApplyProcess)vCurrent).setLocalName(LocIn.getNames(),LocOut.getNames());
	
	for( ListIterator l = vCalls.listIterator(); l.hasNext() ; ){
	    ((Call)l.next()).createProcess(vCurrent,iProcessFactory);
	}
	currentProcess.addProcess(vCurrent);
	update(vCurrent);
    } //end proc

    public void setUp() {
	if( getProcess().getTransmorpher().findTransformation(getRef()) == null ){
	    System.err.println("warning : unknown process : "+getRef());
	}
	update(null);
    } //end setUp
   
    /** This is the main problematic aspect: 
	There is a possible recursion in Process calls: they should thus be stacked for working correctly
    **/
    public void generateJavaCode(Writer file) throws IOException {
	String r = getAttributes().getStringParameter("ref");
	Transmorpher tr = getProcess().getTransmorpher() ;
	fr.fluxmedia.transmorpher.graph.Process processDef = ((fr.fluxmedia.transmorpher.graph.Process)tr.findTransformation(r));
	file.writeln();
	file.writeln(4,"//------------------ Generation of ApplyProcess "+getId()+" -------------------");
	generatePorts(file);
	generateParameters(file);
	file.writeln(4,"tmCurrentProcess = tmProcessFactory.newApplyProcess(portIn,portOut,\""+getType()+"\",tmParameters,tmAttributes);");
	file.writeln(4,"tmCurrentProcess.setName(\""+getId()+"\");");
	file.writeln();
	file.writeln(4,"tmCurrentCompositeProcess.addProcess(tmCurrentProcess);");
	file.writeln();
	// JE: Here we stack the tmCurrentCompositeProcess
	file.writeln(4,"tmProcessStack.push((Object)tmCurrentCompositeProcess);");
	file.writeln(4,"tmCurrentCompositeProcess = (TProcessComposite)tmCurrentProcess;");
	file.writeln();
	// JE: Change of port names ?
	file.writeln(4,"LocIn = new String["+ processDef.inPorts().getNames().length+"];");
	for(int i=0;i<processDef.inPorts().length();i++){
	    file.writeln(4,"LocIn["+i+"]=\""+processDef.inPorts().getNames()[i]+"\";");
	}
	file.writeln(4,"LocOut = new String["+processDef.outPorts().getNames().length+"];");
	for(int i=0;i<processDef.outPorts().length();i++){
	    file.writeln(4,"LocOut["+i+"]=\""+processDef.outPorts().getNames()[i]+"\";");
	}
	file.writeln(4,"((TApplyProcess)tmCurrentProcess).setLocalName(LocIn,LocOut);");
	file.writeln(4,"//------------------ Generation of process body for "+getId()+" -------------------");
	for( ListIterator l = processDef.getCalls().listIterator(); l.hasNext() ; ){
	    ((Call)l.next()).generateJavaCode(file);
	}
	file.writeln(4,"//----------------------- End of ----process body for "+getId()+" ---------------------");
	// JE: Here we unstack the tmCurrentCompositeProcess
	file.writeln(4,"tmCurrentCompositeProcess = (TProcessComposite)tmProcessStack.pop();");
    } //end proc

    public void generateLocalName(Writer file) {};
   
} //end class
