/**
 * $Id: ApplyRuleset.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph call node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.engine.TProcess;

public class ApplyRuleset extends ApplyImpl {

         
    public ApplyRuleset(){
	this((String)null,(String)null);
    }
    
    public ApplyRuleset(String n, String t){
	this(n,t,null,null,null);
    }
    
    
    public ApplyRuleset (String n, String t, Process p){
	this(n,t,p,null,null);
    }

    public ApplyRuleset (String n, String t, Process p, String r){
	this(n, t, p, r, "one-pass/top-down/apply-once");
    }

    public ApplyRuleset (String n, String t, Process p, String r, String s){
	super(n, t, p, r, 1, 1);
	// JE I still do not like to have these attributes in parameters
	if(s!=null)
	    iAttributes.setParameter("strategy",s);
    }
    

  /** Propagate the nullify of its sole output to all the input. */
  public void retroNull( PortList out, PortList in) {
      nullified = out.getPort(0).getChannel().nullifiedP();
      in.getPort(0).getChannel().setVisited();
      if ( nullified ) in.getPort(0).getChannel().nullify();
      visited = true;

      update(null);
  }
  
  public final void setStrategy( String s ){
    iAttributes.setParameter("strategy",s);
    update(null);
    
  }
    
  public final String getStrategy(){
    return iAttributes.getStringParameter("strategy");
  }

  public final void generateXML (){
    System.out.print("      <apply-ruleset id=\""+id+"\" ref=\""+getRef()+"\" strategy=\""+getStrategy()+"\" ");
    System.out.print("in=\"");
    inPorts.generateXML();
    System.out.print("\" out=\"");
    outPorts.generateXML();
    if ( parameters.isEmpty() ){
      System.out.println("\"/>");
    } else {
      System.out.println("\">");
      parameters.generateXMLCalls();
      System.out.println("      </apply-ruleset>");
    }
  }

   public  final void createProcess(TProcessComposite currentProcess, TProcessFactory iProcessFactory)
    {
	String[] vPortIn = inPorts().toStringList();
	String[] vPortOut = outPorts().toStringList();
	// get the name corresponding to the generated stylesheet and pas it in parameter
	String ref = getAttributes().getStringParameter("ref");
	Transmorpher t = getProcess().getTransmorpher() ;
	String f = t.findTransformation(ref).getAttributes().getStringParameter("file");
	// static because generate by Transmorpher from the ruleset name
	getParameters().setParameter("file",f);
	TProcess vCurrent = (TProcess)TProcessFactory.newApplyRuleset(vPortIn,vPortOut,
								 getAttributes().getStringParameter("type"),
								 getParameters(),
								 getAttributes());
	currentProcess.addProcess(vCurrent);

	update(vCurrent);
    } //end proc

  public final void setUp(){
    if( getProcess().getTransmorpher().findTransformation(getRef()) == null ){
      System.err.println("warning : unknown ruleset : "+getRef());
    } //end if

    update(null);
    
  } //end proc
    
  
    public void generateJavaCode(Writer file) throws IOException {
	file.writeln();
	file.writeln(4,"//------------------ Generation of applyRuleSet "+getId()+" -------------------");
	generatePorts(file);
	// Fetch the file to be processed by XSLT (this is very xsltish)
	String ref = getAttributes().getStringParameter("ref");
	Transmorpher tr = getProcess().getTransmorpher() ;
	String f = tr.findTransformation(ref).getAttributes().getStringParameter("file");
	// Added just for being generated afterwards
	getParameters().setParameter("file",f);
	generateParameters(file);
	file.writeln(4,"tmCurrentProcess = tmProcessFactory.newApplyRuleset(portIn,portOut,\""+getType()+"\",tmParameters,tmAttributes);");
	file.writeln(4,"tmCurrentProcess.setName(\""+getId()+"\");");
	file.writeln("");
	file.writeln(4,"tmCurrentCompositeProcess.addProcess(tmCurrentProcess);");
    } //end proc
  
} //end class
