/**
 * $Id: CallImpl.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph call node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.util.Enumeration;
import java.util.Observable;

import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.StringParameters;
import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.action.*;

public abstract class CallImpl extends Observable implements Call {

    String id = null;
    Parameters parameters = null;
    StringParameters iAttributes = null;
    
    PortList inPorts = null;
    PortList outPorts = null;
    boolean visited = false;
    boolean nullified = false;
    Process process = null;
    
  public CallImpl (String n, String t, Process p, int numIn, int numOut) {
    id = n;
    process = p;
    inPorts = new PortList(numIn);
    outPorts = new PortList(numOut);
    parameters = new Parameters();
    iAttributes = new StringParameters();
    if(t!= null) 
	iAttributes.setParameter("type",t);
  }

  /** Prints the XML expression of the call */
    public abstract void generateXML();

  /** Prints the Java code for the execution of the call */
    public abstract void generateJavaCode(Writer file) throws IOException;

    
    public String toString(){
	return id;
    }
    
    public String getId(){
	return id;
    }
    
    /** 
	A modification of the id involved a modification in the process Struct Storage.
     */
    public void setId(String n){

	if(process != null){
	    
	    if((!id.equals(n))){
		
		//remove in the storage struct the call with the old id key
		process.getCalls().remove(id);
		
		//add in the storage struct the call with the new id key 
		process.getCalls().add(n , this);
		
	    }
	    id = n;
	}
	else{
	    //to be thought
	    id = n; 
	}
	update(null);
    }
    
  public String getType(){
    return iAttributes.getStringParameter("type");
  }
  
  public void setType(String t){
    iAttributes.setParameter("type",t);
    update(null);
  }

  public void setParameters(Parameters param){
    parameters = param;
    update(null);
  }

  public Parameters getParameters(){
    return parameters;
  }
  
  public StringParameters getAttributes(){
    return iAttributes;
  }

  public Process getProcess(){
    return process;
  }
    
    public void addOut(Port port){
	addOut(outPorts.length(),port);
    }
    
    public void addIn(Port port){
	addIn(inPorts.length(),port);
    } 
    
    public void addIn(int index,Port port){
	inPorts.setPort(index,port);
	update(new TmActionImpl(TmActionImpl.ADD,this,port,null,null));
    } 
    
    public void addOut(int index,Port port){
	outPorts.setPort(index,port);
	update(new TmActionImpl(TmActionImpl.ADD,this,port,null,null));
    }

  public PortList inPorts(){
    return inPorts;
  }
  
  public PortList outPorts(){
    return outPorts;
  }
  
  /** clear the "visited" mark */
    public void clearVisited() {
      visited = false;
      update(null);
  }
  
  /** clear the "null" mark */
  public void clearNull() {
      nullified = false;
      update(null);
  }
  
  /** propagate the null values of the input to the output.
   * by default, mark the channel as visited */
  public void retroNull() {
      visited = true;
      update(null);
  }

  public void setUp(){
      // By default do nothing
      
  }
  
  public void generatePorts(Writer file) throws IOException {
    file.writeln(4,"portIn = new String["+this.inPorts().getNames().length+"];");
    for(int i=0;i<inPorts.length();i++){
      file.writeln(4,"portIn["+i+"]=\""+this.inPorts().getNames()[i]+"\";");
    } //end for
    file.writeln(4,"portOut = new String["+this.outPorts().getNames().length+"];");
    for(int i=0;i<outPorts.length();i++){
      file.writeln(4,"portOut["+i+"]=\""+this.outPorts().getNames()[i]+"\";");
    } //end for
  } //end proc
   
  public void generateParameters(Writer file) throws IOException {
    file.writeln();
    file.writeln(4,"tmAttributes = new StringParameters(); ");
    for( Enumeration e = getAttributes().getNames() ; e.hasMoreElements(); ){
      String key = (String)e.nextElement();
      String val = (String)getAttributes().getParameter(key);
      file.writeln(4,"tmAttributes.setParameter(\""+ key +"\",\""+ val +"\" );");
    } //end for
    file.writeln(4,"tmParameters = new Parameters(); ");
    for( Enumeration e = getParameters().getNames() ; e.hasMoreElements(); ){
      String key = (String)e.nextElement();
      String val = (String)getParameters().getParameter(key);
      if (((getProcess().getTransmorpher().getMain())instanceof Servlet)&&(key.equals("file"))){
	      file.writeln(4,"tmParameters.setParameter(\""+ key +"\",root+\""+ val +"\" );");
      }
      else
	      file.writeln(4,"tmParameters.setParameter(\""+ key +"\",\""+ val +"\" );");
    } //end for
    file.writeln();
  } //end proc

      // By default do nothing
    public void createProcess(TProcessComposite composite, TProcessFactory pFactory){};


    public void update(Object object){
	setChanged();
	notifyObservers(object);
    }

} //end call impl
