/**
 * $Id: Channel.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph channel representation
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.io.Serializable;
import java.util.Observable;

import fr.fluxmedia.transmorpher.utils.Writer;

public class Channel extends Observable implements Serializable {

    Port in = null;
    Port out = null;
    String name = null;
    boolean visited = false;
    boolean nullified = false;
    Process process = null;

    public Channel(){
    }
    
    public Channel(String n, Process p){
	this( n,p,null,null);
    }
    
    public Channel(String n, Process p, Port i, Port o){
	name=n;
	process=p;
	setIn(i);
	setOut(o);
    }
    
    public String toString(){
	return name;
    }     
    
    public Port in() { return in;}
    public Port out() {return out;}

    public void setIn(Port p) {
	if (in != null)
	    in.setChannel(null);
	
	in = p; 
	
	if (in != null)
	    in.setChannel(this);
    }
    public void setOut(Port p) {
	if (out != null)
	    out.setChannel(null);
	
	out = p; 
	
	if (out != null)
	    out.setChannel(this);
    }
    
    public void setName(String newName){
	
	//A channel's change name must also change associated port name
	if(process!=null)
	    {
		process.getAllChannels().remove(this.name);
		process.getAllChannels().put(newName,this);		
	    }
	if(in != null)
	    in.setName(newName);
	if(out != null)
	    out.setName(newName);
	
	name = newName;
    }
    
    public String getName(){return name;}
    public Process getProcess(){ return process;}
    public void clearNull() { nullified = false;}
    public void nullify() { nullified = true;}
    public void clearVisited() { visited = false;}
    public void setVisited() { visited = true;}
    public boolean visitedP() { return visited;}
    public boolean nullifiedP() { return nullified;}
    
} //end class
