/**
 * $Id: Generate.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph generator node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

/** CAUTION, This will have to change the parameter cannot always be a file */

package fr.fluxmedia.transmorpher.graph ;

import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.engine.TProcess;

public class Generate extends CallImpl {

    public Generate(){
	this(null,null);
    }

    public Generate(String n,String t){
	this(n,t,(String)null);
    }

     public Generate(String n,String t, String file){
	this(n,t,null,0,file);
     }
    
    public  Generate(String n,String t, Process p){
	this(n,t,p,0);
    }

    public  Generate(String n,String t,  String file, Process p){
	this(n,t,p,0,file);
    }
    
    public Generate (String n, String t, Process p, int out){
	this(n, t, p, out, null);
    }
    
   
    /** For compatibility with pre-1.0 Transmorpher */
    public Generate (String n, String t, Process p, int out, String file){
	super(n, t, p, 0, out);
	/** This is for legacy code */
	if (file != null ) parameters.setParameter("file",file);
    }
    
    /** A generate can be nullified but it will propagate to nothing */
    public void retroNull( PortList out, PortList in) {
      nullified = out.getPort(0).getChannel().nullifiedP();
      visited = true;
      update(null);
    }

    public void setFile(String uri){
	if(uri!=null)
	     parameters.setParameter("file",uri);
    }

    
    public void generateXML (){
	System.out.print("      <generate id=\""+id+"\" type=\""+getType()+"\"");
	System.out.print(" out=\"");
	outPorts.generateXML();
	if ( parameters.isEmpty() ){
	    System.out.println("\"/>");
	} else {
	    System.out.println("\">");
	    parameters.generateXMLCalls();
	    System.out.println("      </generate>");
	}
    }//end generate xml
 
    public final void createProcess(TProcessComposite currentProcess, TProcessFactory iProcessFactory){
     
	String[] vPortOut = outPorts().toStringList();	
	//System.out.println(" vPortOut "+ vPortOut);
	//System.out.println("[ExecutionStructure]Creation d'un TReader "+ getParameters());
	TProcess vCurrent = (TProcess)iProcessFactory.newGenerator(vPortOut,
								   getAttributes().getStringParameter("type"),
								   getParameters(),
								   getAttributes());
	currentProcess.addProcess(vCurrent);
	
	update(vCurrent);
    } //end proc
    
    public void generateJavaCode(Writer file) throws IOException {
	file.writeln();
	file.writeln(4,"//------------------ Generation of Generator "+getId()+" -------------------");
	generatePorts(file);
	generateParameters(file);
	file.writeln(4,"tmCurrentProcess = tmProcessFactory.newGenerator(portOut,\""+getType()+"\",tmParameters,tmAttributes);");
	file.writeln(4,"tmCurrentProcess.setName(\""+getId()+"\");");
	file.writeln("");
	file.writeln(4,"tmCurrentCompositeProcess.addProcess(tmCurrentProcess);");
    } //end proc
} //end class
