/**
 * $Id: Main.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph main process definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;
import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;

public class Main extends MainProcess {

  public Main(String name,Transmorpher t){
    super(name,t,0,0);
  }

  /** Prints the XML expression of the main */
  public void generateXML(){
    System.out.println("  <main name=\""+getName()+"\">");
    parameters.generateXMLDeclarations();
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
      ((Call)l.next()).generateXML();
    } //end for
    System.out.println("  </main>");
    System.out.println();
  }

  public TProcessComposite createMainProcess(TProcessFactory iProcessFactory) {
    String[] vPortIn = inPorts().toStringList();
    String[] vPortOut = outPorts().toStringList();
    TProcessComposite vCurrent = (TProcessComposite)TProcessFactory.newMain(vPortIn,vPortOut,getParameters());
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
	Call call = (Call)l.next();
	call.createProcess(vCurrent,iProcessFactory);
    }
    return vCurrent ;
  }

    public void generateImport(Writer iFile,LinearIndexedStruct externs) throws IOException {
	iFile.writeln("import fr.fluxmedia.transmorpher.application.CommandLineArgument;");
    }
 
    public void generateClassHeader(Writer iFile) throws IOException {
	iFile.writeln();
	iFile.writeln("public class "+getName()+" {");
    }
 
    public void generateMainHeader(Writer iFile) throws IOException {
	iFile.writeln(2,"public static void main(String[] argc) throws Exception {");
	iFile.writeln(4,"tmProcessFactory = new TProcessFactory(newDefexterns(),debug_mode);");
	iFile.writeln(4,"tmProcessStack = new Stack();");
	iFile.writeln(4,"vSR = new SystemResources();");
    }
 
    public void generateReadParameters(Writer iFile) throws IOException {
	iFile.writeln(4,"CommandLineArgument arg = new CommandLineArgument();");
	iFile.writeln(4,"arg.argsAnalyse(argc);");
	iFile.writeln(4,"debug_mode = arg.getDebugMode();");
	iFile.writeln(4,"boolean thread_mode = arg.getThreadMode();");
	iFile.writeln(4,"tmParameters = arg.getParameters();");
    }

    public final void generateBody(Writer iFile,LinearIndexedStruct externs) throws IOException {
	Parameters param = getParameters();
	iFile.writeln(4,"Parameters p = new Parameters();");
	for( Enumeration e = param.getNames() ; e.hasMoreElements(); ){
	    String key = (String)e.nextElement();
	    iFile.writeln(4,"p.setParameter(\""+key+"\", \""+(String)param.getParameter(key)+"\");");
	}
	iFile.writeln(4,"tmCurrentCompositeProcess = (TMain)tmProcessFactory.newMain(new String[0], new String[0], p);");
	for( ListIterator l = getCalls().listIterator(); l.hasNext() ; ){
	    ((Call)l.next()).generateJavaCode(iFile);
	}
	// JE: warning, reuse of parameters
	iFile.writeln(4,"tmCurrentCompositeProcess.bindParameters(p);");
	iFile.writeln(4,"tmCurrentCompositeProcess.execProcess(thread_mode);");
    } //end proc

    public void generateEnd(Writer iFile) throws IOException {
	iFile.writeln(4,"if (debug_mode>0) {");
	iFile.writeln(6,"end = System.currentTimeMillis();");
	iFile.writeln(6,"System.err.println(\">> End of execution (\"+(end-begin)+\")\");}");
	iFile.writeln(2,"} //end main");
	iFile.writeln("} //end class");
    }

} //end class
