/**
 * $Id: MainProcess.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph processes definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;
import java.util.Stack;
import java.util.Date;
import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.utils.Version;
import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.engine.TApplyProcess;
import fr.fluxmedia.transmorpher.engine.TProcess;
import fr.fluxmedia.transmorpher.engine.TMain;

public abstract class MainProcess extends fr.fluxmedia.transmorpher.graph.Process {

    public MainProcess(String n, Transmorpher t, int nin, int nout){
	super(n,t,nin,nout);
    }
    
    public abstract TProcessComposite createMainProcess(TProcessFactory iProcessFactory);
    
    public void generateJavaCode(Writer iFile,LinearIndexedStruct externs) throws IOException {
	iFile.writeln("/** Generated by "+Version.NAME+" "+Version.RELEASE);
	iFile.writeln("    See "+Version.URL);
	Date date = new Date();
	iFile.writeln("    "+date+" **/");
	iFile.writeln();
	iFile.writeln("// import Java classes");
	iFile.writeln("import java.util.Stack;");
	iFile.writeln();
	generateImport(iFile,externs);
	iFile.writeln("// import transmorpher classes");
	iFile.writeln("import fr.fluxmedia.transmorpher.engine.*;");
	iFile.writeln("import fr.fluxmedia.transmorpher.utils.*;");
	iFile.writeln();
	generateClassHeader(iFile);
	iFile.writeln(4," // ressource manager");
	iFile.writeln(4,"static SystemResources vSR = null;");
	iFile.writeln();
	iFile.writeln(4,"//global variables for port");
	iFile.writeln(4,"static String[] portIn  = null ;");
	iFile.writeln(4,"static String[] portOut = null;");
	iFile.writeln(4,"static String[] LocIn  = null ;");
	iFile.writeln(4,"static String[] LocOut = null;");
	iFile.writeln();
	iFile.writeln(4," // global variable for the processFactory");
	// BUG: This is not really compiling since stuff are created at runtime!
	iFile.writeln(4,"static TProcessFactory tmProcessFactory = null;");
	iFile.writeln();
	iFile.writeln(4,"private static LinearIndexedStruct newDefexterns(){");
	iFile.writeln(6,"LinearIndexedStruct defs = new LinearIndexedStruct();");
	Enumeration keys = externs.getKeys();
	while(keys.hasMoreElements()){
		String name = (String) keys.nextElement();
	    iFile.writeln(6,"defs.add(\""+name+"\", (Object)\""+(String)externs.get(name)+"\");");
	};
	iFile.writeln(6,"return defs;");
	iFile.writeln(4,"}");
	iFile.writeln(4," // stack of composite processes");
	iFile.writeln(4,"static Stack tmProcessStack = null;");
	iFile.writeln();
	iFile.writeln(4," // global variables for attributes and parameters");
	// Default values should be that of Transmorpher
	iFile.writeln(4,"static int debug_mode = 0;");
	iFile.writeln(4,"static Parameters tmParameters = null ;");
	iFile.writeln(4,"static StringParameters tmAttributes = null;");
	iFile.writeln();
	generateMainHeader(iFile);
	//temporaire
	iFile.writeln(4,"double begin = System.currentTimeMillis();");
	iFile.writeln(4,"double end = 0;");
	iFile.writeln(4,"TProcessComposite tmCurrentCompositeProcess = null;");
	iFile.writeln(4,"TProcess tmCurrentProcess = null;");
	generateReadParameters(iFile);
	generateBody(iFile,externs);
	generateEnd(iFile);
    }
    
    public abstract void generateImport(Writer iFile,LinearIndexedStruct externs) throws IOException;
    public abstract void generateClassHeader(Writer iFile) throws IOException;
    public abstract void generateMainHeader(Writer iFile) throws IOException;
    public abstract void generateReadParameters(Writer iFile) throws IOException;
    public abstract void generateBody(Writer iFile,LinearIndexedStruct externs) throws IOException;
    public abstract void generateEnd(Writer iFile) throws IOException;

} //end class
