/**
 * $Id: Merge.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph call node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer ;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;
import fr.fluxmedia.transmorpher.engine.TProcess;

public class Merge extends CallImpl {

     public Merge(){
	this(null,null);
    }
    
    public Merge(String n, String t){
	this(n,t,null);
    }
    
    public Merge(String n, String t, Process p){
	this(n,t,p,0);
    }
    
    public Merge (String n, String t, Process p, int in){
	super(n, t, p, in, 1);
    }
      
    public PortList inPorts(){
	return inPorts;
    }
   

  /** Propagate the nullify of its sole output to all the input */
  public void retroNull( PortList out, PortList in) {
      nullified = out.getPort(0).getChannel().nullifiedP();
      for ( int i=1; i != in.length() ; i++) {
          in.getPort(i).getChannel().setVisited();
	  if ( nullified ) in.getPort(i).getChannel().nullify();
      } //end for
      visited = true;
      
      //notify observer
      update(null);
  }
  
  public void generateXML (){
    System.out.print("      <merge id=\""+id+"\" type=\""+getType()+"\" ");
    System.out.print("in=\"");
    inPorts.generateXML();
    System.out.print("\" out=\"");
    outPorts.generateXML();
    System.out.println("\"/>");
  }

  public final void createProcess(TProcessComposite currentProcess, TProcessFactory iProcessFactory) {
    String[] vPortIn = inPorts().toStringList();
    String[] vPortOut = outPorts().toStringList();
    TProcess vCurrent = (TProcess)TProcessFactory.newConnector(vPortIn,vPortOut,
						     getAttributes().getStringParameter("type"),
						     getParameters() );
    currentProcess.addProcess(vCurrent);
    
    update(vCurrent);
  } //end proc

 
  public void generateJavaCode(Writer file) throws IOException {
      file.writeln();
      file.writeln(4,"//------------------ Generation of Merge "+getId()+" -------------------");
      generatePorts(file);
      generateParameters(file);
      file.writeln(4,"tmCurrentProcess = tmProcessFactory.newConnector(portIn,portOut,\""+getType()+"\",tmParameters);");
      file.writeln(4,"tmCurrentProcess.setName(\""+getId()+"\");");
      file.writeln();
      file.writeln(4,"tmCurrentCompositeProcess.addProcess(tmCurrentProcess);");
  } //end proc
}
