/**
 * $Id: Ruleset.java,v 1.2 2002/11/07 08:13:26 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph processes definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Date;
import java.io.File;
import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.utils.Version;
import fr.fluxmedia.transmorpher.utils.TMException;
import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.graph.Rules.*;

public class Ruleset extends TransformationImpl implements Namespaced {

    ArrayList rules = null;
    ArrayList namespaces = null;

  public Ruleset(String n, Transmorpher t){
    super(n,"Ruleset",t,1,1);
    rules = new ArrayList(10);
    namespaces = new ArrayList(10);
  }

  public void addRule( Rule r ){
    rules.add((Object)r);
  }

  public void addNamespace( Namespace n ){
    namespaces.add((Object)n);
  }

  public void insertRule( Rule r, int i ){
    rules.add(i, (Object)r);
  }

  /** Prints the XML expression of the process */
  public void generateXML(){
    System.out.println("  <ruleset name=\""+name+"\" type=\""+type+"\">");
    for( int i = 0; i < namespaces.size(); i++ ){
      ((Namespace)namespaces.get(i)).generateXML();
    }
    parameters.generateXMLDeclarations();
    for( int i = 0; i < rules.size(); i++ ){
      ((Rule)rules.get(i)).generateXML();
    }
    System.out.println("  </ruleset>");
    System.out.println();
  };

  /** Prints the Java code for the execution of the call */
  public void generateJavaCode(Writer file,LinearIndexedStruct externs){}

    /** generate the stylesheet for the Query (in the correct file) */
    public void generateXSLTStylesheet(String reloc) throws TMException, IOException {
	String vPath = getTransmorpher().getFile().getAbsoluteFile().getParent();
	if (vPath==null) {vPath = ".";}
	else vPath = vPath.replace('\\','/');
	if( reloc != null ) vPath = reloc;
	//System.out.println("[RuleSEt] vPath = " + vPath +" Reloc = "+ reloc);
	generateXSLTCode(new Writer(vPath+"/"+name+".xsl"));
	iAttributes.setParameter("file",vPath+"/"+name+".xsl");
    }

    /** Prints the Java code for the execution of the call */
    public void generateXSLTCode(Writer file) throws TMException, IOException {
	file.writeln("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	file.writeln("<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\"");
	for( int i = 0; i < namespaces.size(); i++ ){
	    ((Namespace)namespaces.get(i)).generateXSLTCode(file);
	}
	file.writeln(" xmlns:regexp=\"http://fr.fluxmedia.transmorpher.stdlib.RegularExpression\">");
	file.writeln("");
	file.writeln("<!-- ************************************************************");
	file.writeln("     Generated by "+Version.NAME+" "+Version.RELEASE);
	Date date = new Date();
	file.writeln("     "+date);
	file.writeln("     See "+Version.URL);
	file.writeln("     ************************************************************ -->");
	for( Enumeration e = parameters.getNames() ; e.hasMoreElements(); ){
	    String key = (String)e.nextElement();
	    file.writeln(2,"<xsl:param "+key+">"+parameters.getParameter(key)+"</xsl:param>");
	}
	file.writeln("");
	file.writeln("  <!-- Copying the root and its attributes -->");
	file.writeln("  <xsl:template match=\"/\">");
	file.writeln("    <xsl:apply-templates/>");
	file.writeln("  </xsl:template>");
	file.writeln("  <xsl:template match=\"@*\">");
	file.writeln("    <xsl:attribute name=\"{name()}\"><xsl:value-of select=\".\"/>");
	file.writeln("    </xsl:attribute>");
	file.writeln("  </xsl:template>");
	file.writeln("");
	file.writeln("  <!-- Copying all elements and attributes -->");
	file.writeln("  <xsl:template match=\"*\">");
	file.writeln("    <xsl:copy>");
	file.writeln("      <xsl:apply-templates select=\"*|@*|text()\"/>");
	file.writeln("    </xsl:copy>");
	file.writeln("  </xsl:template>");
	file.writeln("");
	file.writeln("<!-- ************************************************************ -->");
	file.writeln("<!--     End of the general section, here begins the stylesheet   -->");
	file.writeln("<!-- ************************************************************ -->");
	file.writeln("");
	for( int i = 0; i < rules.size(); i++ ){
	    ((Rule)rules.get(i)).generateXSLTCode(file);
	}//end for
	file.writeln("</xsl:stylesheet>") ;
	file.close();
  }

} //end class
