/**
 * $Id: Servlet.java,v 1.2 2002/11/08 09:42:54 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph servlet definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;
import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;

public class Servlet extends MainProcess {

  public Servlet(String name, Transmorpher t){
    super(name, t, 0, 0);
  }
  
  public void retroNull() {
  }

  /** Prints the XML expression of the servlet */
  public void generateXML(){
    System.out.println("  <servlet name=\""+name+"\">");
    parameters.generateXMLDeclarations();
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
      ((Call)l.next()).generateXML();
    }
    System.out.println("  </servlet>");
    System.out.println();
  }

  public TProcessComposite createMainProcess(TProcessFactory iProcessFactory) {
    String[] vPortIn = inPorts().toStringList();
    String[] vPortOut = outPorts().toStringList();
    TProcessComposite vCurrent = (TProcessComposite)TProcessFactory.newServlet(vPortIn,vPortOut,getParameters());
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
	((Call)l.next()).createProcess(vCurrent,iProcessFactory);
    }
    return vCurrent ;
  }

    public void generateImport(Writer iFile,LinearIndexedStruct externs) throws IOException {
	iFile.writeln("//import Servlet classes");
	iFile.writeln("import java.io.*;");
	iFile.writeln("import javax.servlet.*;");
	iFile.writeln("import javax.servlet.http.*;");
	iFile.writeln("import java.util.*;");

	iFile.writeln();
    }
 
    public void generateClassHeader(Writer iFile) throws IOException {
	iFile.writeln("public class "+getName()+" extends HttpServlet {");
    }

    public void generateMainHeader(Writer iFile) throws IOException {
	iFile.writeln(2,"static ServletContext context;");
	iFile.writeln(2,"static Parameters servletParameters = null;");
	iFile.writeln(2,"static String root;");
	iFile.writeln(2,"public void init(ServletConfig config) throws ServletException{");
	iFile.writeln(4,"try{");
	iFile.writeln(6,"context = config.getServletContext();");
	iFile.writeln(6,"root = context.getRealPath(\"/\");");
	iFile.writeln(6,"vSR= new SystemResources();");
	iFile.writeln(6,"tmProcessStack = new Stack();");
	iFile.writeln(6,"tmProcessFactory = new TProcessFactory(newDefexterns(),debug_mode);");
	iFile.writeln(4,"} catch (Exception TransformException) {");
	iFile.writeln(4,"}");
	iFile.writeln(2,"}");
	iFile.writeln();
	iFile.writeln(2,"protected void doGet(HttpServletRequest req, HttpServletResponse res)");
	iFile.writeln(4,"          throws ServletException, IOException {");
	iFile.writeln(4,"   doPost(req,res);");
	iFile.writeln(2,"}");
	iFile.writeln();
	iFile.writeln(2,"protected void doPost(HttpServletRequest req, HttpServletResponse res)");
	iFile.writeln(4,"          throws ServletException, IOException {");
	iFile.writeln();
    }
 
    public void generateReadParameters(Writer iFile) throws IOException {
	// JE: Here are only considered the parameters that have been declared
	iFile.writeln(6,"servletParameters = new Parameters();");
	
	iFile.writeln();
	iFile.writeln(6,"try{");
		// See if this must not be modified
	iFile.writeln(8,"res.setContentType(\"text/html\");");
	//iFile.writeln(8,"PrintWriter out = res.getWriter();");
	iFile.writeln(8,"System.setOut(new PrintStream(res.getOutputStream()));");
    }
 
    public void generateBody(Writer iFile,LinearIndexedStruct externs) throws IOException {
	    // JE: I strongly suspect that this should take place in the ReadParameters stuff
	Parameters param = getParameters();
	for( Enumeration e = param.getNames() ; e.hasMoreElements(); ){
	    String key = (String)e.nextElement();
	    iFile.writeln(8,"servletParameters.setParameter(\""+key+"\", \""+(String)param.getParameter(key)+"\");");
	}
	for( Enumeration e = getParameters().getNames() ; e.hasMoreElements(); ){
	    Object key = e.nextElement();
	    iFile.writeln(6,"if ( req.getParameter(\""+key+"\") != null ){");
	    iFile.writeln(8,"servletParameters.setParameter(\""+key+"\",req.getParameter(\""+key+"\"));");
	    iFile.writeln(6,"};");
	} //end for
	iFile.writeln(8,"tmCurrentCompositeProcess = (TServlet)tmProcessFactory.newServlet(new String[0], new String[0], servletParameters);");
	for( ListIterator l = getCalls().listIterator(); l.hasNext() ; ){
	    ((Call)l.next()).generateJavaCode(iFile);
	}
	// JE: warning, reuse of parameters
	iFile.writeln(8,"setoutput(tmCurrentCompositeProcess,res);");

	iFile.writeln(8,"tmCurrentCompositeProcess.bindParameters(servletParameters);");
	iFile.writeln(8,"tmCurrentCompositeProcess.execProcess(false);");
	
	
    /* String reloc = null;	
    for (ListIterator l = transformations.listIterator(); l.hasNext();) {
      Transformation t = ((Transformation)l.next());
      if (t instanceof Ruleset) ((Ruleset)t).generateXSLTStylesheet(reloc);
      else if (t instanceof Query) ((Query)t).generateXSLTStylesheet(reloc);
    } //end for
 	  iProcessFactory = new TProcessFactory(defexterns,debug);
	  iExecutionStructure = createMainProcess(iProcessFactory);
    tmParametersd.bindParameters(p); // ou l'inverse
						graph.exec(p); */
    }
 
    public void generateEnd(Writer iFile) throws IOException {
	iFile.writeln(4,"if (debug_mode>0) {");
	iFile.writeln(6,"end = System.currentTimeMillis();");
	iFile.writeln(6,"System.err.println(\">> End of execution (\"+(end-begin)+\")\");}");
	iFile.writeln(4,"} catch (Exception e) {");
	iFile.writeln(6,"System.out.println(e);");
	iFile.writeln(6,"e.printStackTrace(System.out);");
	iFile.writeln(4,"}");
	iFile.writeln(2,"} //end main");
	generateSetOutput(iFile);
	iFile.writeln("} //end class");
    }
		
	public void generateSetOutput(Writer iFile)throws IOException{
		iFile.writeln(2,"protected void setoutput(TProcessComposite tmCurrentCompositeProcess,HttpServletResponse res){");
		iFile.writeln(4,"for (Enumeration e = tmCurrentCompositeProcess.getProcessBasic().elements() ; e.hasMoreElements() ;) {");
		iFile.writeln(6,"boolean test=true;");
		iFile.writeln(6,"TProcess_Impl process=(TProcess_Impl)e.nextElement();");
		iFile.writeln(6,"if (process instanceof TSerializer){");
		iFile.writeln(8,"for (Enumeration a = servletParameters.getNames() ; a.hasMoreElements() ;){");
		iFile.writeln(10,"String name = (String)a.nextElement();");
		iFile.writeln(10,"if (process.getParameter(name)!=null){");
		iFile.writeln(12,"if (process.getParameter(name).equals(servletParameters.getParameter(name))){");
		iFile.writeln(14,"test=true;");
		iFile.writeln(12,"}");
		iFile.writeln(12,"else{");
		iFile.writeln(14,"test=false;");
		iFile.writeln(12,"}");
		iFile.writeln(10,"}");
		iFile.writeln(10,"else{");
		iFile.writeln(12,"test=false;");
		iFile.writeln(10,"}");
		iFile.writeln(8,"}");
		iFile.writeln(8,"if (test){");
		iFile.writeln(10,"((TSerializer)process).printStdout(true);");
		iFile.writeln(10,"res.setContentType((String)process.getParameter(\"content-type\"));");
		iFile.writeln(8,"}");
		iFile.writeln(8,"else");
		iFile.writeln(10,"((TSerializer)process).printNull(true);");
		iFile.writeln(6,"}");
		iFile.writeln(4,"}");
		iFile.writeln(2,"}");
	}
} //end class
