/**
 * $Id: TransformationImpl.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph call node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.util.Observable;
import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.StringParameters;
import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.utils.Writer;

import fr.fluxmedia.transmorpher.action.TmActionImpl;

public abstract class TransformationImpl extends Observable implements Transformation {

    String name;
    PortList inPorts = null;
    PortList outPorts = null;
    Parameters parameters = null;
    StringParameters iAttributes = null;
    Transmorpher transmorpher = null;
    String type;

    public TransformationImpl(String n, String t, Transmorpher tm)
    {
	this(n,t,tm,0,0);
    }
    
    public TransformationImpl( String n, String t, Transmorpher tm,  int nin, int nout ){
	name = n;
	type = t;
	transmorpher = tm;
	inPorts = new PortList(nin);
	outPorts = new PortList(nout);
	parameters = new Parameters();
	iAttributes = new StringParameters();
	iAttributes.setParameter("type",t);
    }
    
    public String getName(){
	return name;
    }
    
    /**
       When the process name change, we must change the key in transmorpher storage structure.
    */
    public void setName(String n){
	
	if(transmorpher != null )
	    {
		//remove in the storage struct the transformation with the old id key
		transmorpher.getTransformations().remove(name);
		//add in the storage struct the transformation with the new id key
		transmorpher.getTransformations().add(n,this);
	    }
	update(new TmActionImpl(TmActionImpl.MODIFY,this,name,name,n));
	name = n;
    }
    
    public Transmorpher getTransmorpher(){
	return transmorpher;
    }
    
    public String getType(){
	return type;
    }
    
    public void setType(String t){
	type = t;
	update(null);
    }
    
    public void setParameters(Parameters p){
	parameters = p;	
	update(p);
    }
    
    public Parameters getParameters(){
	return parameters;
    }
    public StringParameters getAttributes(){
	return iAttributes;
    }

    public void addIn(Port p) {
	addIn(inPorts.length(),p);
    }

    public void addIn( int i, Port p) {
	inPorts.setPort(i,p);
	update(new TmActionImpl(TmActionImpl.ADD,this,p,null,null));
    }
    
    public PortList inPorts(){
	return inPorts;
    }

    public void addOut(Port p) {
	addOut(outPorts.length(),p);
    }
    
    public void addOut( int i, Port p) {
	outPorts.setPort(i,p);
	update(new TmActionImpl(TmActionImpl.ADD,this,p,null,null));
    }
    
    public PortList outPorts(){
	return outPorts;
    }
    
  /** Prints the XML expression of the call */
    public abstract void generateXML();
    
  /** Prints the Java code for the execution of the call */
    public abstract void generateJavaCode(Writer file, LinearIndexedStruct externs) throws IOException;
    
    /** This is the trivial case if all the output is null then all the input are null **/
    public void retroNull(PortList out, PortList in) {
	boolean nullified = true;
	for ( int i=0; nullified && (i != out.length()); i++ ) {
	    if( !out.getPort(i).getChannel().nullifiedP() ) nullified = false;
	}
	for ( int i=0; i != in.length(); i++ ) {
	    in.getPort(i).getChannel().setVisited();
	    if( nullified ) in.getPort(i).getChannel().nullify();
	}
    }


    public void update(Object object){
	setChanged();
	notifyObservers(object);
    }
    
    
} //end class
