/*
 * $Id: Transformer.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph (JAXP) transformer definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;
import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TProcessComposite;

public class Transformer extends MainProcess {

  public Transformer(String name, Transmorpher t){
    super(name,t,1,1);
  }

  public Transformer(Transmorpher t){
    this("transformer", t);
  }

  private String XMLName(){ return "transformer";}

  public TProcessComposite createMainProcess(TProcessFactory iProcessFactory) {
    String[] vPortIn = inPorts().toStringList();
    String[] vPortOut = outPorts().toStringList();
    TProcessComposite vCurrent = (TProcessComposite)TProcessFactory.newTransformer(vPortIn,vPortOut,getParameters());
    for( ListIterator l = calls.listIterator(); l.hasNext() ; ){
	((Call)l.next()).createProcess(vCurrent,iProcessFactory);
    }
    return vCurrent ;
  }
    public void generateImport(Writer iFile,LinearIndexedStruct externs) throws IOException {
	iFile.writeln("import java.util.Properties;");
	iFile.writeln("//import JAXP classes");
	iFile.writeln("import javax.xml.transform.*;");
	iFile.writeln("import javax.xml.transform.stream.*;");
	iFile.writeln();
    }
 
    public void generateClassHeader(Writer iFile) throws IOException {
	iFile.writeln("public class "+getName()+" extends javax.xml.transform.Transformer {");
    }


    public void generateMainHeader(Writer iFile) throws IOException {
	// JE: Here, this should not be main but:
	iFile.writeln(2,"public void transform(Source in, Result out){");
	//iFile.writeln(2,"public static void main(String[] args){");
	iFile.writeln(4,"tmProcessFactory = new TProcessFactory(newDefexterns(),debug_mode);");
	iFile.writeln(4,"tmProcessStack = new Stack();");
	iFile.writeln(4,"vSR = new SystemResources();");
	iFile.writeln();
    }
 
    public void generateReadParameters(Writer iFile) throws IOException {
	// JE: Here are only considered the parameters that have been declared
	for( Enumeration e = getParameters().getNames() ; e.hasMoreElements(); ){
	    Object key = e.nextElement();
	    // JE: Here I must find the code corresponding to the transformer
	    iFile.writeln(4,"if ( req.getParameter(\""+key+"\") != null ){");
	    iFile.writeln(6,"tmParameters.setParameter(\""+key+"\",req.getParameter(\""+key+"\"));");
	    iFile.writeln(4,"};");
	} //end for
	iFile.writeln();
	// iFile.writeln(4,"res.setContentType(\"text/plain\");");
	// iFile.writeln(4,"PrintWriter out = res.getWriter();");
	iFile.writeln();
    }
 
    public final void generateBody(Writer iFile,LinearIndexedStruct externs) throws IOException {
	Parameters param = getParameters();
	iFile.writeln(4,"Parameters p = new Parameters();");
	for( Enumeration e = param.getNames() ; e.hasMoreElements(); ){
	    String key = (String)e.nextElement();
	    iFile.writeln(4,"p.setParameter(\""+key+"\", \""+(String)param.getParameter(key)+"\");");
	}
	iFile.writeln(4,"tmCurrentCompositeProcess = (TMain)tmProcessFactory.newMain(new String[0], new String[0], p);");
	for( ListIterator l = getCalls().listIterator(); l.hasNext() ; ){
	    ((Call)l.next()).generateJavaCode(iFile);
	}
	// JE: warning, reuse of parameters
	iFile.writeln(4,"tmCurrentCompositeProcess.bindParameters(p);");
	iFile.writeln(4,"try{");
	iFile.writeln(6,"tmCurrentCompositeProcess.execProcess(false);");
	iFile.writeln(4,"}catch(Exception e){}");
    } //end proc

    public void generateEnd(Writer iFile) throws IOException {
	iFile.writeln(4,"if (debug_mode>0) {");
	iFile.writeln(6,"end = System.currentTimeMillis();");
	iFile.writeln(6,"System.err.println(\">> End of execution (\"+(end-begin)+\")\");}");
	iFile.writeln(2,"} //end main");
	generateTransformerMethod(iFile);
	iFile.writeln("} //end class");
	
    }
		
    public void generateTransformerMethod(Writer iFile) throws IOException {
	iFile.writeln(4,"public void clearParameters(){");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public ErrorListener getErrorListener(){");
	iFile.writeln(6,"return null;");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public Properties getOutputProperties(){");
	iFile.writeln(6,"return null;");
	iFile.writeln(4,"}");
			
	iFile.writeln(4,"public String getOutputProperty(String name){");
	iFile.writeln(6,"return null;");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public Object getParameter(String name){");
	iFile.writeln(6,"return null;");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public URIResolver getURIResolver(){");
	iFile.writeln(6,"return null;");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public void setErrorListener(ErrorListener listener){");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public void setOutputProperties(Properties ofromat){");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public void setOutputProperty(String name,String value){");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public void setParameter(String name, Object value){");
	iFile.writeln(4,"}");
	
	iFile.writeln(4,"public void setURIResolver(URIResolver resolver){");
	iFile.writeln(4,"}");
	
    }

} //end class
