/**
 * $Id: Transmorpher.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.transmorpher.graph;

// import Java classes

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Observable;

import java.io.IOException;
import java.io.File;
import java.io.PrintStream;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

import org.xml.sax.SAXException;

import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.parsers.ParserConfigurationException;

// Transmorpher imports

import fr.fluxmedia.transmorpher.action.*;
import fr.fluxmedia.transmorpher.utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.Version;
import fr.fluxmedia.transmorpher.utils.TMException;
import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.engine.TProcessFactory ;
import fr.fluxmedia.transmorpher.engine.TProcessComposite ;
import fr.fluxmedia.transmorpher.parser.ProcessParser ;
import fr.fluxmedia.transmorpher.parser.FMParser ;

public class Transmorpher extends Observable {

 /** The name of the Transmorpher sheet */
  protected String name = null ;
    
  /** The filename of the Transmorpher sheet */
  protected File href = null ;
    
  /** The main process of the Transmorpher sheet */
  protected MainProcess main = null ;
    
  /** the list of Processes */
  protected LinearIndexedStruct transformations = null ;

  /** the version to be used */
  String version = null ;

  /* the execution structure */
  TProcessComposite iExecutionStructure = null;
  
  /* the process factory used for generating the execution structure */
  TProcessFactory iProcessFactory = null;

  /** the list of imports must be a list of Transmorphers
      once the imports have been parsed. In such a case the URL can be found
      in its href attribute */
  protected LinkedList imports = null ;

  /** the list of includes */
  protected LinkedList includes = null ;

  /** the list of declared names of components */
    protected LinearIndexedStruct defexterns = null ;

    protected int debug = 0;
    
    protected String reloc = "";
		
    protected boolean opt = false;    

    protected boolean useThread=false;
    
  public Transmorpher(int d){
  // JE: name might be compulsory...
    debug = d;
    transformations = new LinearIndexedStruct();
    defexterns = new LinearIndexedStruct();
    imports = new LinkedList();
    includes = new LinkedList();
  }

  public Transmorpher(String n, String v, int d, boolean o){
    this(d);
    opt = o;
    name = n;
    href = new File( name+".xml" );
    version = v;
    if (!version.equals(Version.VERSION) ) 
	System.err.println("Warning : bad version "+version+" instead of "+Version.VERSION);
  }

    public Transmorpher(String n, String v, int d,String _reloc, boolean o){
	this(n,v,d,o);
	
	if (_reloc!=null) reloc=_reloc;
    }
	
    // This is a Transmorpher inheriting some externs...
  public Transmorpher(String n, String v, int d,String _reloc, boolean o, Transmorpher t){
    this(n, v, d,_reloc, o);
    defexterns = t.getDefexterns();
  }

  public final String getName(){
    return name;
  }
    
  public final void setName(String n){
    name = n;
    setChanged();
    //notifyObservers(new TmAddTransformationAction(t));
    notifyObservers();
  }
    /**
     * Get the value of debug.
     * @return value of debug.
     */
    public int getDebug() {return debug;}
    
    /**
     * Set the value of debug.
     * @param v  Value to assign to debug.
     */
    public void setDebug(int  v) {this.debug = v;}
    
    /**
     * 
     * @return true if using thread.
     */
    public boolean isThread() {return useThread;}
    
    /**
     * 
     * @param v  Value to assign to debug.
     */
    public void useThread(boolean  thread) {this.useThread = thread;}
    
    
    
  public final File getFile(){
    return href;
  }
    
  public final void setFile(File u){
      href = u;
  }
    
  public final String getVersion(){
    return version;
  }
    
  public final void setVersion(String v){
    version = v;
  }
    
  public final boolean optimizedP(){
    return opt;
  }
    
  public final void setOptimized(boolean b){
    opt = b;
  }
    
  public final MainProcess getMain(){
    return main;
  }
    
    public LinearIndexedStruct getTransformations(){
	return transformations;
    }
    
    public final Transformation findTransformation(String name){
	Transformation result = (Transformation)transformations.get(name);
	if (result != null) return result;
	for (ListIterator l = imports.listIterator(0); ( l.hasNext() && (result != null ) );) {
	    result = ((Transmorpher)l.next()).findTransformation( name );
	} //end for
	return result;
    }
    
    public final Transformation getTransformation(String name){
	return (Transformation)transformations.get(name);
    }
    
    public final void addTransformation(String name, Transformation t){
	// JE: If it already exists one == error (this will cause filename problems)
	transformations.add(name, (Object)t);
	setChanged();
	notifyObservers(new TmActionImpl(TmActionImpl.ADD,this,t,null,null));
	//notifyObservers(t);
    }
    
  public final void removeTransformation( Transformation trans ) {
      transformations.remove( trans.getName() );
      // Check that all the processes defined here do not use the transformation.
      // If so, suppress it from the transformation p.deleteCall( c )
      if ( !(trans instanceof MainProcess) ) {
	 for (ListIterator l = transformations.listIterator(); l.hasNext();) {
	   Transformation t = ((Transformation)l.next());
	   if (t instanceof Process) ((Process)t).deleteCallsTo( trans.getName() );
	 } // end for
       } // end if
      // Take care of the main
      if ( main == (MainProcess)trans ) main = null;
      // the transformation should not be reachable anymore
      
      setChanged();
      notifyObservers(new TmActionImpl(TmActionImpl.REMOVE,this,trans,null,null));

  }
    
  public final String getExtern(String name){
    return (String)defexterns.get(name);
  }

  public final void addExtern(String name, String cl, String impl, boolean def){
      if ( getExtern(name) != null ){
	  System.err.println("Error: "+name+" external already declared");
      } else {
	  defexterns.add(name, (Object)cl);
      }
  }
    
  public final void removeExtern( String name ) {
      defexterns.remove( name );
      // Check that all the processes defined here do not use the external call.
      // If so, suppress it from the transformation p.deleteCall( c )
      for (ListIterator l = transformations.listIterator(); l.hasNext();) {
	   Transformation t = ((Transformation)l.next());
	   if (t instanceof Process) ((Process)t).deleteCallsTo( name );
      } // end for
  }
    
  public final LinearIndexedStruct getDefexterns() { return defexterns; }
    
    /** just put the import URI in the linked list **/
  public final void reserveImport(String i){
      imports.add((Object)i);
  }

    /** replace the import URI by the actual import **/
    public final void recordImport(String i, Transmorpher t){
	imports.set(imports.indexOf((Object)i),(Object)t);
    }

    /** This is an internal method for dynamically adding imports **/
    public final void addImport(String i) throws ParserConfigurationException, SAXException, IOException {
      // JE: The String i must be transformed in a URL that could be loaded!
      FMParser iProcessParser = new ProcessParser(0);
      Transmorpher timp = iProcessParser.Newparse(i);
      // JE: Check if there will not be a name conflict
      imports.add((Object)timp);
  }

  public final LinkedList getImports(){
      return imports;
  }
    
  public final void addInclude(String i){
    includes.add((Object)i);
  }

  public final LinkedList getIncludes(){
    return includes;
  }
    
  public final void setMain(MainProcess p){
    main = p;
  }

  public final void retroNull(){
    main.retroNull();
  }

  public final void save() throws TMException {
      if ( href != null && href.canWrite() ){
	  if ( href.exists() ) href.renameTo(new File(href+"~"));
	  PrintStream stdout = System.out;
	  try {
	      System.setOut( new PrintStream(
				 new BufferedOutputStream(
				       new FileOutputStream( href ))));
	      generateXML();
	      System.out.close();
	  } catch ( FileNotFoundException e ){
	      throw new TMException("Cannot save in file "+href);
	  } finally {
	      System.setOut(stdout);
	  }
      } else throw new TMException("Cannot save in file "+href);
  }

  public final void saveAll() throws TMException {
      // JE: How to avoid saving twice ?
      for (ListIterator l = imports.listIterator(0); l.hasNext();)
	  ((Transmorpher)l.next()).saveAll();
      for (ListIterator l = includes.listIterator(0); l.hasNext();)
	  ((Transmorpher)l.next()).saveAll();
      save();
  }

  public final void generateXML(){
      // This does not generate the imported files
      System.out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      System.out.println("<!DOCTYPE transmorpher SYSTEM \"http://transmorpher.inrialpes.fr/dtd/transmorpher.dtd\">");
      System.out.println("<transmorpher name=\""+name+"\"");
      System.out.println("              version=\""+version+"\"");
      System.out.println("              optimized=\""+opt+"\"");
      System.out.println("              xmlns=\"http://transmorpher.fluxmedia.fr/1.0\"");
      System.out.println("              xmlns:regexp=\"http://fr.fluxmedia.transmorpher.regexp.RegularExpression\">");
      System.out.println();
      
      Enumeration keys = defexterns.getKeys();
      while(keys.hasMoreElements()){
	  String name = (String) keys.nextElement();
	  System.out.println("<defextern name=\""+name+"\" class=\""+(String)defexterns.get(name)+"\"/>");
      }	
      for (ListIterator l = imports.listIterator(0); l.hasNext();) {
	  System.out.println("<import href=\""+((Transmorpher)l.next()).getName()+"\"/>");
      } //end for
      for (ListIterator l = includes.listIterator(0); l.hasNext();) {
	  System.out.println("<include href=\""+((Transmorpher)l.next()).getName()+"\"/>");
      }
      for (ListIterator l = transformations.listIterator(); l.hasNext();) {
	  Transformation t = ((Transformation)l.next());
	  if ( t  != main ){
	      t.generateXML();
	  } //end if
      } //end for
      if ( main != null ) main.generateXML();
      System.out.println("</transmorpher>");
  } //end generate XML
    

  /** Generates Java code for the transformation **/
  public final void generateExec(String _reloc)
    throws TransformerException,
           TransformerConfigurationException,
	   TMException,
           SAXException,
           IOException {
      if(_reloc!=null) reloc=_reloc;
      for (ListIterator l = transformations.listIterator(); l.hasNext();) {
	  Transformation t = ((Transformation)l.next());
	  if (t instanceof Ruleset) ((Ruleset)t).generateXSLTStylesheet(reloc);
	  else if (t instanceof Query) ((Query)t).generateXSLTStylesheet(reloc);
      } //end for
      if ( getMain() != null ){
	  iProcessFactory = new TProcessFactory(defexterns,debug);
	  iExecutionStructure = getMain().createMainProcess(iProcessFactory);
	  // JE: This should include the generation of the import/include!
	  // So far, this is not absolutely easy
      } else {
	  throw new TMException("Cannot generate execution structure for mainless Transformation");
      }
  } //end generateExec

    public final void generateJavaCode(String _reloc) throws IOException, TMException {
	String iCompilDirectory = null;
	Writer iFile = null;
	if(_reloc!=null) reloc=_reloc;
	for (ListIterator l = transformations.listIterator(); l.hasNext();) {
	    Transformation t = ((Transformation)l.next());
	    if (t instanceof Ruleset) ((Ruleset)t).generateXSLTStylesheet(reloc);
	    else if (t instanceof Query) ((Query)t).generateXSLTStylesheet(reloc);}
	try {
	    iCompilDirectory = getFile().getAbsoluteFile().getParent();
	    if( reloc != null) iCompilDirectory = reloc;
	    else iCompilDirectory = getFile().getAbsoluteFile().getParent();
	    File dir = new File(iCompilDirectory);
	    dir.mkdir(); // directly tries to create the directory that might exist
	    iCompilDirectory+=File.separator;
	    iFile = new Writer(iCompilDirectory+getMain().getName()+".java");
	    getMain().generateJavaCode(iFile,defexterns);
	} finally {
	    iFile.close();
	};
    } //end generate JavaCode

  public final void exec(Parameters p) throws Exception {
      if ( iExecutionStructure != null ){
	  iExecutionStructure.bindParameters(p);
	  iExecutionStructure.execProcess(useThread);
      }
  } //end run

}//end class
