/**
 * $Id: DefAtt.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher default attribute rule
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph.Rules ;

import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer;

public class DefAtt implements Rule {

  public String match = null;
  public String value = null;
  public String context = null;

  public DefAtt(String m, String v){
    super();
    match = m;
    value = v;
  }

  public DefAtt(String m, String v, String c){
    this(m, v);
    context = c;
  }

  /** Prints the XML expression of the rule */
  public void generateXML(){
    System.out.print("    <defatt match=\""+match+"\"");
    if ( context != null ) System.out.print(" context=\""+context+"\"");
    System.out.println("value=\""+value+"/>");
  }

  public void generateXSLTCode(Writer file) throws IOException {
    file.writeln(2,"<!-- Default attributes for "+match+": "+value+" -->");
    file.write(2,"<xsl:template match=\"");
    if( context != null ) file.write(context+"/");
    file.writeln("@"+ match+"\">");
    file.writeln(4,"<xsl:attribute name=\""+value+"\">");
    file.write("<xsl:value-of select=\"@"+match+"\"/>");
    file.writeln(4,"</xsl:attribute>");
    file.writeln(2,"</xsl:template>");
    file.writeln("");
  }

  /** Prints the XSLT code inside a ModTag template
   This is the only way for a default attribute rule to work*/
  public void generateInsideXSLTCode(Writer file) throws IOException {
    file.writeln(6,"<!-- Default attributes for "+match+": "+value+" -->");
    file.writeln(6,"<xsl:if test=\"not(@\""+match+")\">");
    file.writeln(8,"<xsl:attribute name=\""+match+"\">");
    file.write(value);
    file.writeln(8,"</xsl:attribute>");
    file.writeln(6,"</xsl:if>");
    file.writeln("");
  }

}
