/**
 * $Id: ModTag.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph rule node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.graph.Rules ;

import java.util.ArrayList;
import java.io.IOException;

import fr.fluxmedia.transmorpher.utils.Writer;
import fr.fluxmedia.transmorpher.utils.TMException;

public class ModTag implements Rule {

  public ArrayList rules = null;
  public String match = null;
  public String context = null;

  public ModTag(String m){
    super();
    match = m;
    rules = new ArrayList(10);
  }

  public ModTag(String m, String c){
    this(m);
    context = c;
  }

  public void addRule( Rule r ){
    rules.add((Object)r);
  }

  /** Prints the XML expression of the rule */
  public void generateXML(){
    System.out.print("    <modtag match=\""+match+"\"");
    if ( context != null ) System.out.print(" context=\""+context+"\"");
    System.out.println(">");
    for( int i = 0; i < rules.size(); i++ ){
      ((Rule)rules.get(i)).generateXML();
    }
    System.out.println("</modtag>");
  }

    public void generateXSLTCode(Writer file) throws TMException, IOException {
    file.writeln(2,"<!-- Modifying element "+match+" -->");
    file.write(2,"<xsl:template match=\"");
    if( context != null ) file.write(context+"/");
    file.writeln(match+"\">");
    file.writeln(4,"<xsl:copy>");
    for( int i = 0; i < rules.size(); i++ ){
      ((Rule)rules.get(i)).generateInsideXSLTCode(file);
    }
    file.writeln(6,"<xsl:apply-templates select=\"*|@*|text()\"/>");
    file.writeln(4,"</xsl:copy>");
    file.writeln(2,"</xsl:template>");
    file.writeln("");
    for( int i = 0; i < rules.size(); i++ ){
      Rule r = (Rule)rules.get(i);
      if ( (r instanceof RemAtt) || (r instanceof MapAtt) ){
        RemAtt rr = (RemAtt)r;
        file.writeln(2,"<!-- Removing attributes "+rr.match
                   +" on elements "+rr.context+" -->");
        file.writeln(4,"  <xsl:template match=\""+rr.context+"/@"+rr.match+"\"/>");
        file.writeln(2,"");
      }
    }
    file.writeln("");
  }

  /** Prints the XSLT code inside a ModTag template */
  public void generateInsideXSLTCode(Writer file) throws TMException {
   throw new TMException("[ModTag]generateInsideXSLTCode : not yet implemented");
  }

}
