/*
 * $Id: ReadFile.java,v 1.1 2002/11/06 14:08:21 serge Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.transmorpher.stdlib;

// Imported Transmorpher classes

import fr.fluxmedia.transmorpher.engine.TProcessFactory;
import fr.fluxmedia.transmorpher.engine.TReader;
import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.TMException;
import fr.fluxmedia.transmorpher.utils.StringParameters;
import fr.fluxmedia.transmorpher.utils.SystemResources;

// Imported Java classes

import java.io.*;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.lang.Runnable;
import java.net.*;
import java.util.Hashtable;

// Imported SAX and JAXP classes

import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 *
 * This class provide a basic process which is
 * able to read an XML file
 * reads a file which coordinates (local path or URL) is give in the
 * file parameter of the generate call, parse it and generate a SAX event
 * flow to the next handler
 *
 *@author    Laurent.Tardif@inrialpes.fr
 *@since     jdk 1.3 && SAX 2.0
 */

public final class ReadFile extends TReader {

    /**
     *Constructor for the ReadFile object
     *
     *@param  pOut               Description of the Parameter
     *@param  pParam             Description of the Parameter
     *@param  pStaticAttributes  Description of the Parameter
     *@exception  SAXException   Description of the Exception
     *@exception  IOException    Description of the Exception
     */
    public ReadFile(String[] pOut, Parameters pParam, StringParameters pStaticAttributes) throws SAXException, IOException {
	super(pOut, pParam, pStaticAttributes);
	try {
	    
	    SAXParserFactory parserFactory = SAXParserFactory.newInstance();
	    parserFactory.setFeature(NAMESPACES_FEATURE_ID, DEFAULT_NAMESPACES);
	    parserFactory.setFeature(NAMESPACES_PREFIXES_FEATURE_ID, DEFAULT_NAMESPACES_PREFIXES);
	    parserFactory.setFeature(VALIDATION_FEATURE_ID, DEFAULT_VALIDATION);
	    parserFactory.setFeature(SCHEMA_VALIDATION_FEATURE_ID, DEFAULT_SCHEMA_VALIDATION);
	    iReader = (parserFactory.newSAXParser()).getXMLReader();
	} catch (Exception e) {
	    System.out.println("[ReadFile]Exception :" + e);
	    System.out.println("[ReadFile] " + SystemResources.getProperty("XML_READER"));
	}//end try
    }
    
    /**
     *  Description of the Method
     *
     *@exception  IOException   if an IO operations failed
     *@exception  SAXException  can wrap others exceptions
     */
    public void read() throws IOException, SAXException {
	InputSource inSource;
	try {
	    if (iFilename != null) {
		if (iFilename.startsWith("http://")) {
		    URL url = new URL(iFilename);
		    URLConnection urlConnect = url.openConnection();
		    InputStream inStream = urlConnect.getInputStream();
		    inSource = new InputSource(inStream);
		    inSource.setSystemId(iFilename);
		    iReader.parse(inSource);
		} else {
		    File file = new File(iFilename);
		    inSource = new InputSource(new FileInputStream(file));
		    inSource.setSystemId(file.toURL().toString());
		    iReader.parse(inSource);
		}
	    } else {
		iReader.parse(standardInput);
	    }
	} catch (IOException e) {
	    // JE: There is a problem here either we report the error, 
	    // either we send the empty document! And if we report it,
	    // this must be by raising an exception
	    System.err.println("File : " + iFilename + " can't be found.");
	    // Send the empty document on error!?
	    iReader.getContentHandler().startDocument();
	    iReader.getContentHandler().endDocument();
	}
    }
}

