/**
 * $Id: StdOut.java,v 1.2 2002/11/07 14:31:11 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
 * This class offer a serializer mechanisme. A way to finish the transphormation process
 * bye writing XML output file
 *
 *@since     jdk 1.3 && SAX 2.0
 *@author    laurent tardif@inrialpes.fr
 *@date      10 02 2001
 */

package fr.fluxmedia.transmorpher.stdlib;

import fr.fluxmedia.transmorpher.engine.TSerializer;
import fr.fluxmedia.transmorpher.utils.Parameters;
import fr.fluxmedia.transmorpher.utils.StringParameters;

import fr.fluxmedia.transmorpher.utils.TMException;
import fr.fluxmedia.transmorpher.utils.TMException;
import fr.fluxmedia.transmorpher.utils.TMRuntimeException;
import fr.fluxmedia.transmorpher.utils.TMRuntimeException;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;

import org.xml.sax.ContentHandler;

// Imported SAX classes

import org.xml.sax.SAXException;

/**
 *  Description of the Class
 *
 *@author    triolet
 */
public final class StdOut extends TSerializer {

	/**
	 *  the handler for the serialization
	 */
	protected TransformerHandler handler;

	/**
	 *  a factory to create a transformer handler
	 */
	protected SAXTransformerFactory tfactory = null;

	/**
	 *Constructor for the StdOut object
	 *
	 *@param  pIn                     in port of this object
	 *@param  pParam             the parameters of this component
	 *@param  pStaticAttributes  static attributes of tjis component
	 *@exception  TMRuntimeException  Description of the Exception
	 */
	public StdOut(String[] pIn, Parameters pParam, StringParameters pStaticAttributes) throws TMRuntimeException {
		super(pIn, pParam, pStaticAttributes);
		try {
			handler = getTransformerFactory().newTransformerHandler();
		} catch (Exception e) {
			throw new TMRuntimeException(e, "[StdOut] Cannot create object");
		}
	}

	/**
	 *  Gets the transformerFactory attribute of the StdOut object
	 *
	 *@return    The transformerFactory value
	 */
	protected SAXTransformerFactory getTransformerFactory() {
		if (tfactory == null) {
			tfactory = (SAXTransformerFactory)TransformerFactory.newInstance();
		}
		return tfactory;
	}

	/**
	 *  Sets the outputStream attribute of the StdOut object
	 */
	public void setOutputStream() {
		handler.getTransformer().setOutputProperties(Op);
		handler.setResult(new StreamResult(System.out));
	}

	/**
	 *  Gets the contentHandler attribute of the StdOut object
	 *
	 *@return    The contentHandler value
	 */
	public ContentHandler getContentHandler() {
		return (ContentHandler)handler;
	}

}

